@@ -85,6 +85,7 @@ GnomeManager::Impl::Impl(GnomeManager* manager, bool test_mode)
   , can_hibernate_(false)
   , pending_action_(shell::Action::NONE)
   , shell_server_(test_mode_ ? testing::DBUS_NAME : shell::DBUS_NAME)
+  , open_sessions_(0)
 {
   shell_server_.AddObjects(shell::INTROSPECTION_XML, shell::DBUS_OBJECT_PATH);
   shell_object_ = shell_server_.GetObject(shell::DBUS_INTERFACE);
@@ -126,6 +127,22 @@ GnomeManager::Impl::Impl(GnomeManager* manager, bool test_mode)
     });
   }
 
+  {
+    dm_proxy_ = std::make_shared<glib::DBusProxy>("org.freedesktop.DisplayManager",
+                                                  "/org/freedesktop/DisplayManager",
+                                                  "org.freedesktop.DisplayManager",
+                                                  G_BUS_TYPE_SYSTEM);
+
+    dm_proxy_->Connect("SessionAdded", sigc::hide(sigc::mem_fun(this, &Impl::UpdateHaveOtherOpenSessions)));
+    dm_proxy_->Connect("SessionRemoved", sigc::hide(sigc::mem_fun(this, &Impl::UpdateHaveOtherOpenSessions)));
+
+    UpdateHaveOtherOpenSessions();
+
+    manager_->have_other_open_sessions.SetGetterFunction([this]() {
+      return open_sessions_ > 1;
+    });
+  }
+
   CallLogindMethod("CanHibernate", nullptr, [this] (GVariant* variant, glib::Error const& err) {
     if (err)
     {
@@ -422,6 +439,21 @@ void GnomeManager::Impl::LockScreen(bool prompt)
   prompt ? manager_->prompt_lock_requested.emit() : manager_->lock_requested.emit();
 }
 
+void GnomeManager::Impl::UpdateHaveOtherOpenSessions()
+{
+  dm_proxy_->GetProperty("Sessions", [this](GVariant* variant) {
+      GVariantIter *sessions;
+      g_variant_get(variant, "ao", &sessions);
+      int open_sessions = g_variant_iter_n_children(sessions);
+
+      if (open_sessions_ != open_sessions)
+      {
+        open_sessions_ = open_sessions;
+        manager_->have_other_open_sessions.changed.emit(open_sessions_);
+      }
+  });
+}
+
 // Public implementation
 
 GnomeManager::GnomeManager()
