@@ -22,7 +22,8 @@
 #define UNITY_INDICATOR_ENTRY_H
 
 #include <iosfwd>
-#include <map>
+#include <unordered_map>
+#include <vector>
 #include <string>
 #include <memory>
 #include <sigc++/signal.h>
@@ -36,13 +37,14 @@ namespace indicator
 
 // The EntryLocationMap is used to map the entry name to the
 // physical location on the screen.
-typedef std::map<std::string, nux::Rect> EntryLocationMap;
+typedef std::unordered_map<std::string, nux::Rect> EntryLocationMap;
 
 class Entry
 {
 public:
   typedef std::shared_ptr<Entry> Ptr;
 
+  Entry(std::string const& id, std::string const& name_hint = "");
   Entry(std::string const& id,
         std::string const& name_hint,
         std::string const& label,
@@ -59,13 +61,17 @@ public:
 
   std::string const& id() const;
   std::string const& name_hint() const;
-  std::string const& label() const;
-  int image_type() const;
-  std::string const& image_data() const;
+
+  void set_image(int type, std::string const& data, bool sensitive, bool visible);
   bool image_visible() const;
   bool image_sensitive() const;
+  int image_type() const;
+  std::string const& image_data() const;
+
+  void set_label(std::string const& label, bool sensitive, bool visible);
   bool label_visible() const;
   bool label_sensitive() const;
+  std::string const& label() const;
 
   void set_active(bool active);
   bool active() const;
@@ -73,6 +79,7 @@ public:
   void set_geometry(nux::Rect const& geometry);
   nux::Rect const& geometry() const;
 
+  void set_priority(int priority);
   int priority() const;
 
   bool visible() const;
@@ -85,14 +92,14 @@ public:
   bool show_now() const;
   void set_show_now(bool show_now);
 
-  void ShowMenu(int x, int y, unsigned int button, unsigned int timestamp);
-  void ShowMenu(unsigned int xid, int x, int y, unsigned int button, unsigned int timestamp);
-  void SecondaryActivate(unsigned int timestamp);
-  void Scroll(int delta);
+  void add_parent(Entry::Ptr const&);
+  void rm_parent(Entry::Ptr const&);
+  std::vector<Entry::Ptr> const& parents() const;
 
-  void setLabel(std::string const& label, bool sensitive, bool visible);
-  void setImage(int type, std::string const& data, bool sensitive, bool visible);
-  void setPriority(int priority);
+  void ShowMenu(int x, int y, unsigned button);
+  void ShowMenu(unsigned int xid, int x, int y, unsigned button);
+  void SecondaryActivate();
+  void Scroll(int delta);
 
   // Signals
   sigc::signal<void> updated;
@@ -100,8 +107,8 @@ public:
   sigc::signal<void, nux::Rect const&> geometry_changed;
   sigc::signal<void, bool> show_now_changed;
 
-  sigc::signal<void, std::string const&, unsigned int, int, int, unsigned int, unsigned int> on_show_menu;
-  sigc::signal<void, std::string const&, unsigned int> on_secondary_activate;
+  sigc::signal<void, std::string const&, unsigned, int, int, unsigned> on_show_menu;
+  sigc::signal<void, std::string const&> on_secondary_activate;
   sigc::signal<void, std::string const&, int> on_scroll;
 
 private:
@@ -122,6 +129,7 @@ private:
   bool active_;
 
   nux::Rect geometry_;
+  std::vector<Entry::Ptr> parents_;
 };
 
 std::ostream& operator<<(std::ostream& out, Entry const& e);
