@@ -23,6 +23,8 @@
 #include <sigc++/sigc++.h>
 #include <memory>
 
+#include <NuxCore/Property.h>
+
 namespace unity
 {
 namespace session
@@ -36,6 +38,8 @@ public:
   Manager() = default;
   virtual ~Manager() = default;
 
+  nux::ROProperty<bool> have_other_open_sessions;
+
   virtual std::string RealName() const = 0;
   virtual std::string UserName() const = 0;
   virtual std::string HostName() const = 0;
