@@ -28,7 +28,6 @@
 #include <NuxGraphics/GraphicsEngine.h>
 #include <Nux/TextureArea.h>
 #include <NuxGraphics/CairoGraphics.h>
-#include <UnityCore/Variant.h>
 
 #include "unity-shared/CairoTexture.h"
 
@@ -40,7 +39,10 @@
 #include "QuicklistMenuItemRadio.h"
 
 #include "unity-shared/Introspectable.h"
+#include "unity-shared/PanelStyle.h"
+#include "unity-shared/DecorationStyle.h"
 #include "unity-shared/UnitySettings.h"
+#include "unity-shared/UScreen.h"
 
 #include "unity-shared/UBusWrapper.h"
 #include "unity-shared/UBusMessages.h"
@@ -50,48 +52,49 @@ namespace unity
 {
 namespace
 {
-  const int ANCHOR_WIDTH = 10.0f;
-  const int TOP_SIZE = 4;
+  const RawPixel ANCHOR_WIDTH  = 10_em;
+  const RawPixel TOP_SIZE      =  4_em;
+
+  const RawPixel ANCHOR_HEIGHT = 18_em;
+  const RawPixel CORNER_RADIUS =  4_em;
+  const RawPixel MAX_HEIGHT  = 1000_em;
+  const RawPixel MAX_WIDTH   = 1000_em;
+
+  const RawPixel LEFT_PADDING_CORRECTION = -1_em;
+  const RawPixel OFFSET_CORRECTION = -1_em;
 }
 
 NUX_IMPLEMENT_OBJECT_TYPE(QuicklistView);
 
-QuicklistView::QuicklistView()
-  : _anchorX(0)
+QuicklistView::QuicklistView(int monitor)
+  : CairoBaseWindow(monitor)
+  , _anchorX(0)
   , _anchorY(0)
   , _labelText("QuicklistView 1234567890")
   , _top_size(TOP_SIZE)
+  , _padding(decoration::Style::Get()->ActiveShadowRadius())
   , _mouse_down(false)
   , _enable_quicklist_for_testing(false)
-  , _anchor_width(10)
-  , _anchor_height(18)
-  , _corner_radius(4)
-  , _padding(13)
-  , _left_padding_correction(-1)
-  , _offset_correction(-1)
   , _cairo_text_has_changed(true)
   , _current_item_index(-1)
 {
   SetGeometry(nux::Geometry(0, 0, 1, 1));
 
-  _left_space = new nux::SpaceLayout(_padding +
-                                     _anchor_width +
-                                     _corner_radius +
-                                     _left_padding_correction,
-                                     _padding +
-                                     _anchor_width +
-                                     _corner_radius +
-                                     _left_padding_correction,
-                                     1, 1000);
-  _right_space = new nux::SpaceLayout(_padding + _corner_radius,
-                                      _padding + _corner_radius,
-                                      1, 1000);
-  _top_space = new nux::SpaceLayout(1, 1000,
-                                    _padding + _corner_radius,
-                                    _padding + _corner_radius);
-  _bottom_space = new nux::SpaceLayout(1, 1000,
-                                       _padding + _corner_radius,
-                                       _padding + _corner_radius);
+  _left_space = new nux::SpaceLayout(RawPixel(_padding + ANCHOR_WIDTH + CORNER_RADIUS + LEFT_PADDING_CORRECTION).CP(cv_),
+                                     RawPixel(_padding + ANCHOR_WIDTH + CORNER_RADIUS + LEFT_PADDING_CORRECTION).CP(cv_),
+                                     1, MAX_HEIGHT.CP(cv_));
+
+  _right_space = new nux::SpaceLayout(_padding.CP(cv_) + CORNER_RADIUS.CP(cv_),
+                                      _padding.CP(cv_) + CORNER_RADIUS.CP(cv_),
+                                      1, MAX_HEIGHT.CP(cv_));
+
+  _top_space = new nux::SpaceLayout(1, MAX_WIDTH.CP(cv_),
+                                    _padding.CP(cv_) + CORNER_RADIUS.CP(cv_),
+                                    _padding.CP(cv_) + CORNER_RADIUS.CP(cv_));
+
+  _bottom_space = new nux::SpaceLayout(1, MAX_WIDTH.CP(cv_),
+                                       _padding.CP(cv_) + CORNER_RADIUS.CP(cv_),
+                                       _padding.CP(cv_) + CORNER_RADIUS.CP(cv_));
 
   _vlayout = new nux::VLayout(TEXT(""), NUX_TRACKER_LOCATION);
   _vlayout->AddLayout(_top_space, 0);
@@ -100,7 +103,7 @@ QuicklistView::QuicklistView()
   _vlayout->AddLayout(_item_layout, 0);
 
   _vlayout->AddLayout(_bottom_space, 0);
-  _vlayout->SetMinimumWidth(140);
+  _vlayout->SetMinimumWidth(RawPixel(140).CP(cv_));
 
   _hlayout = new nux::HLayout(TEXT(""), NUX_TRACKER_LOCATION);
   _hlayout->AddLayout(_left_space, 0);
@@ -123,6 +126,16 @@ QuicklistView::QuicklistView()
   SetAcceptKeyNavFocus(true);
 }
 
+int QuicklistView::CalculateX() const
+{
+  return _anchorX - _padding.CP(cv_);
+}
+
+int QuicklistView::CalculateY() const
+{
+  return _anchorY - (ANCHOR_HEIGHT.CP(cv_) / 2) - _top_size.CP(cv_) - CORNER_RADIUS.CP(cv_) - _padding.CP(cv_);
+}
+
 void
 QuicklistView::RecvStartFocus()
 {
@@ -306,18 +319,6 @@ QuicklistView::RecvKeyPressed(unsigned long    eventType,
   }
 }
 
-QuicklistView::~QuicklistView()
-{
-  for (auto item : _item_list)
-  {
-    // Remove from introspection
-    RemoveChild(item);
-    item->UnReference();
-  }
-
-  _item_list.clear();
-}
-
 void
 QuicklistView::EnableQuicklistForTesting(bool enable_testing)
 {
@@ -333,27 +334,22 @@ void QuicklistView::SetQuicklistPosition(int tip_x, int tip_y)
   {
     if (!_item_list.empty())
     {
-      int offscreen_size = GetBaseY() +
-                           GetBaseHeight() -
-                           nux::GetWindowThread()->GetGraphicsDisplay().GetWindowHeight();
+      auto* us = UScreen::GetDefault();
+      int ql_monitor = us->GetMonitorAtPosition(_anchorX, _anchorY);
+      auto const& ql_monitor_geo = us->GetMonitorGeometry(ql_monitor);
+      int offscreen_size = GetBaseY() + GetBaseHeight() - (ql_monitor_geo.y + ql_monitor_geo.height);
 
       if (offscreen_size > 0)
         _top_size = offscreen_size + TOP_SIZE;
       else
         _top_size = TOP_SIZE;
 
-      int x = _anchorX - _padding;
-      int y = _anchorY - _anchor_height / 2 - _top_size - _corner_radius - _padding;
-
-      SetBaseXY(x, y);
+      SetXY(CalculateX(), CalculateY());
     }
     else
     {
       _top_size = 0;
-      int x = _anchorX - _padding;
-      int y = _anchorY - _anchor_height / 2 - _top_size - _corner_radius - _padding;
-
-      SetBaseXY(x, y);
+      SetXY(CalculateX(), CalculateY());
     }
   }
 }
@@ -409,7 +405,7 @@ void QuicklistView::Draw(nux::GraphicsEngine& gfxContext, bool forceDraw)
 
   gfxContext.PushClippingRectangle(base);
 
-  for (auto item : _item_list)
+  for (auto const& item : _item_list)
   {
     if (item->GetVisible())
       item->ProcessDraw(gfxContext, forceDraw);
@@ -426,17 +422,17 @@ void QuicklistView::PreLayoutManagement()
   int MaxItemWidth = 0;
   int TotalItemHeight = 0;
 
-  for (auto item : _item_list)
+  for (auto const& item : _item_list)
   {
     // Make sure item is in layout if it should be
     if (!item->GetVisible())
     {
-      _item_layout->RemoveChildObject(item);
+      _item_layout->RemoveChildObject(item.GetPointer());
       continue;
     }
     else if (!item->GetParentObject())
     {
-      _item_layout->AddView(item, 1, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FULL);
+      _item_layout->AddView(item.GetPointer(), 1, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FULL);
     }
 
     nux::Size const& text_extents = item->GetTextExtents();
@@ -444,10 +440,10 @@ void QuicklistView::PreLayoutManagement()
     TotalItemHeight += text_extents.height;
   }
 
-  if (TotalItemHeight < _anchor_height)
+  if (TotalItemHeight < ANCHOR_HEIGHT.CP(cv_))
   {
-    int b = (_anchor_height - TotalItemHeight) / 2 + _padding + _corner_radius;
-    int t = b + _offset_correction;
+    int b = (ANCHOR_HEIGHT.CP(cv_) - TotalItemHeight) / 2 + _padding.CP(cv_) + CORNER_RADIUS.CP(cv_);
+    int t = b + OFFSET_CORRECTION.CP(cv_);
 
     _top_space->SetMinimumHeight(t);
     _top_space->SetMaximumHeight(t);
@@ -457,8 +453,8 @@ void QuicklistView::PreLayoutManagement()
   }
   else
   {
-    int b = _padding + _corner_radius;
-    int t = b + _offset_correction;
+    int b = _padding.CP(cv_) + CORNER_RADIUS.CP(cv_);
+    int t = b + OFFSET_CORRECTION.CP(cv_);
 
     _top_space->SetMinimumHeight(t);
     _top_space->SetMaximumHeight(t);
@@ -478,10 +474,10 @@ long QuicklistView::PostLayoutManagement(long LayoutResult)
 
   UpdateTexture();
 
-  int x = _padding + _anchor_width + _corner_radius + _offset_correction;
+  int x = RawPixel(_padding + ANCHOR_WIDTH + CORNER_RADIUS + OFFSET_CORRECTION).CP(cv_);
   int y = _top_space->GetMinimumHeight();
 
-  for (auto item : _item_list)
+  for (auto const& item : _item_list)
   {
     if (!item->GetVisible())
       continue;
@@ -499,7 +495,7 @@ long QuicklistView::PostLayoutManagement(long LayoutResult)
   // has bee set correctly during the layout cycle, but the cairo rendering still need to be adjusted.
   unsigned separator_width = _item_layout->GetBaseWidth();
 
-  for (auto item : _item_list)
+  for (auto const& item : _item_list)
   {
     if (item->GetVisible() && item->GetCairoSurfaceWidth() != separator_width)
     {
@@ -536,7 +532,7 @@ void QuicklistView::RecvItemMouseClick(QuicklistMenuItem* item, int x, int y)
 void QuicklistView::CheckAndEmitItemSignal(int x, int y)
 {
   nux::Geometry geo;
-  for (auto item : _item_list)
+  for (auto const& item : _item_list)
   {
     if (!item->GetVisible())
       continue;
@@ -547,7 +543,7 @@ void QuicklistView::CheckAndEmitItemSignal(int x, int y)
     if (geo.IsPointInside(x, y))
     {
       // An action is performed: send the signal back to the application
-      ActivateItem(item);
+      ActivateItem(item.GetPointer());
     }
   }
 }
@@ -576,18 +572,16 @@ void QuicklistView::RecvItemMouseRelease(QuicklistMenuItem* item, int x, int y)
 
 void QuicklistView::CancelItemsPrelightStatus()
 {
-  for (auto item : _item_list)
-  {
+  for (auto const& item : _item_list)
     item->Select(false);
-  }
 }
 
 void QuicklistView::RecvItemMouseDrag(QuicklistMenuItem* item, int x, int y)
 {
   nux::Geometry geo;
-  for (auto it : _item_list)
+  for (auto const& it : _item_list)
   {
-    int item_index = GetItemIndex(it);
+    int item_index = GetItemIndex(it.GetPointer());
 
     if (!IsMenuItemSelectable(item_index))
       continue;
@@ -659,24 +653,15 @@ void QuicklistView::RecvMouseDownOutsideOfQuicklist(int x, int y, unsigned long
 
 void QuicklistView::RemoveAllMenuItem()
 {
-  for (auto item : _item_list)
-  {
-    // Remove from introspection
-    RemoveChild(item);
-    item->UnReference();
-  }
-
-
-  _item_list.clear();
-
   _item_layout->Clear();
+  _item_list.clear();
   _cairo_text_has_changed = true;
-  nux::GetWindowThread()->QueueObjectLayout(this);
+  QueueRelayout();
 }
 
 void QuicklistView::AddMenuItem(QuicklistMenuItem* item)
 {
-  if (item == 0)
+  if (!item)
     return;
 
   item->sigTextChanged.connect(sigc::mem_fun(this, &QuicklistView::RecvCairoTextChanged));
@@ -686,15 +671,12 @@ void QuicklistView::AddMenuItem(QuicklistMenuItem* item)
   item->sigMouseEnter.connect(sigc::mem_fun(this, &QuicklistView::RecvItemMouseEnter));
   item->sigMouseLeave.connect(sigc::mem_fun(this, &QuicklistView::RecvItemMouseLeave));
   item->sigMouseDrag.connect(sigc::mem_fun(this, &QuicklistView::RecvItemMouseDrag));
+  item->SetScale(cv_->DPIScale());
 
-  _item_list.push_back(item);
-  item->Reference();
-  // Add to introspection
-  AddChild(item);
+  _item_list.push_back(QuicklistMenuItem::Ptr(item));
 
   _cairo_text_has_changed = true;
-  nux::GetWindowThread()->QueueObjectLayout(this);
-  NeedRedraw();
+  QueueRelayout();
 }
 
 void QuicklistView::RenderQuicklistView()
@@ -712,10 +694,10 @@ QuicklistMenuItem* QuicklistView::GetNthItems(int index)
   if (index < (int)_item_list.size())
   {
     int i = 0;
-    for (auto item : _item_list)
+    for (auto const& item : _item_list)
     {
       if (i++ == index)
-        return item;
+        return item.GetPointer();
     }
   }
 
@@ -726,7 +708,7 @@ int QuicklistView::GetItemIndex(QuicklistMenuItem* item)
 {
   int index = -1;
 
-  for (auto it : _item_list)
+  for (auto const& it : _item_list)
   {
     ++index;
 
@@ -746,7 +728,7 @@ QuicklistMenuItemType QuicklistView::GetNthType(int index)
   return QuicklistMenuItemType::UNKNOWN;
 }
 
-std::list<QuicklistMenuItem*> QuicklistView::GetChildren()
+std::list<QuicklistMenuItem::Ptr> QuicklistView::GetChildren()
 {
   return _item_list;
 }
@@ -757,23 +739,23 @@ void QuicklistView::SelectFirstItem()
 }
 
 void ql_tint_dot_hl(cairo_t* cr,
+                    gfloat  scale,
                     gint    width,
                     gint    height,
                     gfloat  hl_x,
                     gfloat  hl_y,
                     gfloat  hl_size,
-                    gfloat* rgba_tint,
-                    gfloat* rgba_hl,
-                    gfloat* rgba_dot)
+                    nux::Color const& tint_color,
+                    nux::Color const& hl_color,
+                    nux::Color const& dot_color)
 {
-  cairo_surface_t* dots_surf    = NULL;
-  cairo_t*         dots_cr      = NULL;
   cairo_pattern_t* dots_pattern = NULL;
   cairo_pattern_t* hl_pattern   = NULL;
 
   // create context for dot-pattern
-  dots_surf = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 4, 4);
-  dots_cr = cairo_create(dots_surf);
+  nux::CairoGraphics dots_surf(CAIRO_FORMAT_ARGB32, 4 * scale, 4 * scale);
+  cairo_surface_set_device_scale(dots_surf.GetSurface(), scale, scale);
+  cairo_t* dots_cr = dots_surf.GetInternalContext();
 
   // clear normal context
   cairo_scale(cr, 1.0f, 1.0f);
@@ -789,10 +771,10 @@ void ql_tint_dot_hl(cairo_t* cr,
 
   // fill path of normal context with tint
   cairo_set_source_rgba(cr,
-                        rgba_tint[0],
-                        rgba_tint[1],
-                        rgba_tint[2],
-                        rgba_tint[3]);
+                        tint_color.red,
+                        tint_color.green,
+                        tint_color.blue,
+                        tint_color.alpha);
   cairo_fill_preserve(cr);
 
   // create pattern in dot-context
@@ -801,15 +783,15 @@ void ql_tint_dot_hl(cairo_t* cr,
   cairo_scale(dots_cr, 1.0f, 1.0f);
   cairo_set_operator(dots_cr, CAIRO_OPERATOR_OVER);
   cairo_set_source_rgba(dots_cr,
-                        rgba_dot[0],
-                        rgba_dot[1],
-                        rgba_dot[2],
-                        rgba_dot[3]);
+                        dot_color.red,
+                        dot_color.green,
+                        dot_color.blue,
+                        dot_color.alpha);
   cairo_rectangle(dots_cr, 0.0f, 0.0f, 1.0f, 1.0f);
   cairo_fill(dots_cr);
   cairo_rectangle(dots_cr, 2.0f, 2.0f, 1.0f, 1.0f);
   cairo_fill(dots_cr);
-  dots_pattern = cairo_pattern_create_for_surface(dots_surf);
+  dots_pattern = cairo_pattern_create_for_surface(dots_surf.GetSurface());
 
   // fill path of normal context with dot-pattern
   cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
@@ -817,8 +799,6 @@ void ql_tint_dot_hl(cairo_t* cr,
   cairo_pattern_set_extend(dots_pattern, CAIRO_EXTEND_REPEAT);
   cairo_fill_preserve(cr);
   cairo_pattern_destroy(dots_pattern);
-  cairo_surface_destroy(dots_surf);
-  cairo_destroy(dots_cr);
 
   // draw highlight
   cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
@@ -830,10 +810,10 @@ void ql_tint_dot_hl(cairo_t* cr,
                                            hl_size);
   cairo_pattern_add_color_stop_rgba(hl_pattern,
                                     0.0f,
-                                    rgba_hl[0],
-                                    rgba_hl[1],
-                                    rgba_hl[2],
-                                    rgba_hl[3]);
+                                    hl_color.red,
+                                    hl_color.green,
+                                    hl_color.blue,
+                                    hl_color.alpha);
   cairo_pattern_add_color_stop_rgba(hl_pattern, 1.0f, 1.0f, 1.0f, 1.0f, 0.0f);
   cairo_set_source(cr, hl_pattern);
   cairo_fill(cr);
@@ -843,17 +823,8 @@ void ql_tint_dot_hl(cairo_t* cr,
 void ql_setup(cairo_surface_t** surf,
               cairo_t**         cr,
               gboolean          outline,
-              gint              width,
-              gint              height,
               gboolean          negative)
 {
-//     // create context
-//     if (outline)
-//       *surf = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
-//     else
-//       *surf = cairo_image_surface_create (CAIRO_FORMAT_A8, width, height);
-//     *cr = cairo_create (*surf);
-
   // clear context
   cairo_scale(*cr, 1.0f, 1.0f);
   if (outline)
@@ -875,8 +846,8 @@ void ql_setup(cairo_surface_t** surf,
 void ql_compute_full_mask_path(cairo_t* cr,
                                gfloat   anchor_width,
                                gfloat   anchor_height,
-                               gint     width,
-                               gint     height,
+                               gfloat   width,
+                               gfloat   height,
                                gint     upper_size,
                                gfloat   radius,
                                guint    pad)
@@ -995,7 +966,7 @@ void ql_compute_mask(cairo_t* cr)
 
 void ql_compute_outline(cairo_t* cr,
                         gfloat   line_width,
-                        gfloat*  rgba_line,
+                        nux::Color const& line_color,
                         gfloat   size)
 {
   cairo_pattern_t* pattern = NULL;
@@ -1007,25 +978,25 @@ void ql_compute_outline(cairo_t* cr,
 
   pattern = cairo_pattern_create_linear(x, y, size, y);
   cairo_pattern_add_color_stop_rgba(pattern, 0.0f,
-                                    rgba_line[0],
-                                    rgba_line[1],
-                                    rgba_line[2],
-                                    rgba_line[3]);
+                                    line_color.red,
+                                    line_color.green,
+                                    line_color.blue,
+                                    line_color.alpha);
   cairo_pattern_add_color_stop_rgba(pattern, offset,
-                                    rgba_line[0],
-                                    rgba_line[1],
-                                    rgba_line[2],
-                                    rgba_line[3]);
+                                    line_color.red,
+                                    line_color.green,
+                                    line_color.blue,
+                                    line_color.alpha);
   cairo_pattern_add_color_stop_rgba(pattern, 1.1f * offset,
-                                    rgba_line[0] * 0.65f,
-                                    rgba_line[1] * 0.65f,
-                                    rgba_line[2] * 0.65f,
-                                    rgba_line[3]);
+                                    line_color.red * 0.65f,
+                                    line_color.green * 0.65f,
+                                    line_color.blue * 0.65f,
+                                    line_color.alpha);
   cairo_pattern_add_color_stop_rgba(pattern, 1.0f,
-                                    rgba_line[0] * 0.65f,
-                                    rgba_line[1] * 0.65f,
-                                    rgba_line[2] * 0.65f,
-                                    rgba_line[3]);
+                                    line_color.red * 0.65f,
+                                    line_color.green * 0.65f,
+                                    line_color.blue * 0.65f,
+                                    line_color.alpha);
   cairo_set_source(cr, pattern);
   cairo_set_line_width(cr, line_width);
   cairo_stroke(cr);
@@ -1035,7 +1006,7 @@ void ql_compute_outline(cairo_t* cr,
 void ql_draw(cairo_t* cr,
              gboolean outline,
              gfloat   line_width,
-             gfloat*  rgba,
+             nux::Color const& color,
              gboolean negative,
              gboolean stroke)
 {
@@ -1046,7 +1017,7 @@ void ql_draw(cairo_t* cr,
   if (outline)
   {
     cairo_set_line_width(cr, line_width);
-    cairo_set_source_rgba(cr, rgba[0], rgba[1], rgba[2], rgba[3]);
+    cairo_set_source_rgba(cr, color.red, color.green, color.blue, color.alpha);
   }
   else
   {
@@ -1066,7 +1037,7 @@ void ql_draw(cairo_t* cr,
 void ql_finalize(cairo_t** cr,
                  gboolean  outline,
                  gfloat    line_width,
-                 gfloat*   rgba,
+                 nux::Color const& color,
                  gboolean  negative,
                  gboolean  stroke)
 {
@@ -1077,7 +1048,7 @@ void ql_finalize(cairo_t** cr,
   if (outline)
   {
     cairo_set_line_width(*cr, line_width);
-    cairo_set_source_rgba(*cr, rgba[0], rgba[1], rgba[2], rgba[3]);
+    cairo_set_source_rgba(*cr, color.red, color.green, color.blue, color.alpha);
   }
   else
   {
@@ -1098,19 +1069,19 @@ void
 ql_compute_full_outline_shadow(
   cairo_t* cr,
   cairo_surface_t* surf,
-  gint    width,
-  gint    height,
+  gfloat  width,
+  gfloat  height,
   gfloat  anchor_width,
   gfloat  anchor_height,
   gint    upper_size,
   gfloat  corner_radius,
   guint   blur_coeff,
-  gfloat* rgba_shadow,
+  nux::Color const& rgba_shadow,
   gfloat  line_width,
   gint    padding_size,
-  gfloat* rgba_line)
+  nux::Color const& rgba_line)
 {
-  ql_setup(&surf, &cr, TRUE, width, height, FALSE);
+  ql_setup(&surf, &cr, TRUE, FALSE);
   ql_compute_full_mask_path(cr,
                             anchor_width,
                             anchor_height,
@@ -1130,10 +1101,9 @@ ql_compute_full_outline_shadow(
 void ql_compute_full_mask(
   cairo_t* cr,
   cairo_surface_t* surf,
-  gint     width,
-  gint     height,
+  gfloat   width,
+  gfloat   height,
   gfloat   radius,
-  guint    shadow_radius,
   gfloat   anchor_width,
   gfloat   anchor_height,
   gint     upper_size,
@@ -1141,9 +1111,9 @@ void ql_compute_full_mask(
   gboolean outline,
   gfloat   line_width,
   gint     padding_size,
-  gfloat*  rgba)
+  nux::Color const&  rgba)
 {
-  ql_setup(&surf, &cr, outline, width, height, negative);
+  ql_setup(&surf, &cr, outline, negative);
   ql_compute_full_mask_path(cr,
                             anchor_width,
                             anchor_height,
@@ -1160,62 +1130,39 @@ void QuicklistView::UpdateTexture()
   if (!_cairo_text_has_changed)
     return;
 
-  int size_above_anchor = -1; // equal to size below
+  SetQuicklistPosition(_anchorX, _anchorY);
+
+  RawPixel size_above_anchor = _item_list.empty() ? RawPixel(-1) : _top_size;
   int width = GetBaseWidth();
   int height = GetBaseHeight();
 
-  if (!_enable_quicklist_for_testing)
-  {
-    if (!_item_list.empty())
-    {
-      int offscreen_size = GetBaseY() +
-                           height -
-                           nux::GetWindowThread()->GetGraphicsDisplay().GetWindowHeight();
-
-      if (offscreen_size > 0)
-        _top_size = offscreen_size + TOP_SIZE;
-      else
-        _top_size = TOP_SIZE;
-
-      size_above_anchor = _top_size;
-      int x = _anchorX - _padding;
-      int y = _anchorY - _anchor_height / 2 - _top_size - _corner_radius - _padding;
-
-      SetBaseX(x);
-      SetBaseY(y);
-    }
-    else
-    {
-      _top_size = 0;
-      size_above_anchor = -1;
-      int x = _anchorX - _padding;
-      int y = _anchorY - _anchor_height / 2 - _top_size - _corner_radius - _padding;
-
-      SetBaseX(x);
-      SetBaseY(y);
-    }
-  }
-
-  float blur_coef         = 6.0f;
+  auto const& deco_style = decoration::Style::Get();
+  float dpi_scale = cv_->DPIScale();
+  float blur_coef = std::round(deco_style->ActiveShadowRadius() * dpi_scale / 2.0f);
 
   nux::CairoGraphics cairo_bg(CAIRO_FORMAT_ARGB32, width, height);
   nux::CairoGraphics cairo_mask(CAIRO_FORMAT_ARGB32, width, height);
   nux::CairoGraphics cairo_outline(CAIRO_FORMAT_ARGB32, width, height);
 
-  cairo_t* cr_bg      = cairo_bg.GetContext();
-  cairo_t* cr_mask    = cairo_mask.GetContext();
-  cairo_t* cr_outline = cairo_outline.GetContext();
+  cairo_surface_set_device_scale(cairo_bg.GetSurface(), dpi_scale, dpi_scale);
+  cairo_surface_set_device_scale(cairo_mask.GetSurface(), dpi_scale, dpi_scale);
+  cairo_surface_set_device_scale(cairo_outline.GetSurface(), dpi_scale, dpi_scale);
 
-  float   tint_color[4]    = {0.0f, 0.0f, 0.0f, HasBlurredBackground() ? 0.60f : 1.0f};
-  float   hl_color[4]      = {1.0f, 1.0f, 1.0f, 0.35f};
-  float   dot_color[4]     = {1.0f, 1.0f, 1.0f, 0.03f};
-  float   shadow_color[4]  = {0.0f, 0.0f, 0.0f, 1.00f};
-  float   outline_color[4] = {1.0f, 1.0f, 1.0f, 0.40f};
-  float   mask_color[4]    = {1.0f, 1.0f, 1.0f, 1.00f};
+  cairo_t* cr_bg      = cairo_bg.GetInternalContext();
+  cairo_t* cr_mask    = cairo_mask.GetInternalContext();
+  cairo_t* cr_outline = cairo_outline.GetInternalContext();
+
+  nux::Color tint_color(0.0f, 0.0f, 0.0f, HasBlurredBackground() ? 0.60f : 1.0f);
+  nux::Color hl_color(1.0f, 1.0f, 1.0f, 0.35f);
+  nux::Color dot_color(1.0f, 1.0f, 1.0f, 0.03f);
+  nux::Color shadow_color(deco_style->ActiveShadowColor());
+  nux::Color outline_color(1.0f, 1.0f, 1.0f, 0.40f);
+  nux::Color mask_color(1.0f, 1.0f, 1.0f, 1.00f);
 
   ql_tint_dot_hl(cr_bg,
-                 width,
-                 height,
+                 dpi_scale,
+                 width / dpi_scale,
+                 height / dpi_scale,
                  width / 2.0f,
                  0,
                  nux::Max<float>(width / 1.6f, height / 1.6f),
@@ -1227,38 +1174,33 @@ void QuicklistView::UpdateTexture()
   (
     cr_outline,
     cairo_outline.GetSurface(),
-    width,
-    height,
-    _anchor_width,
-    _anchor_height,
+    width / dpi_scale,
+    height / dpi_scale,
+    ANCHOR_WIDTH,
+    ANCHOR_HEIGHT,
     size_above_anchor,
-    _corner_radius,
+    CORNER_RADIUS,
     blur_coef,
     shadow_color,
-    1.0f,
+    1.0f * dpi_scale,
     _padding,
     outline_color);
 
   ql_compute_full_mask(
     cr_mask,
     cairo_mask.GetSurface(),
-    width,
-    height,
-    _corner_radius,  // radius,
-    16,             // shadow_radius,
-    _anchor_width,   // anchor_width,
-    _anchor_height,  // anchor_height,
-    size_above_anchor,             // upper_size,
-    true,           // negative,
-    false,          // outline,
-    1.0,            // line_width,
-    _padding,        // padding_size,
+    width / dpi_scale,
+    height / dpi_scale,
+    CORNER_RADIUS,             // radius,
+    ANCHOR_WIDTH,              // anchor_width,
+    ANCHOR_HEIGHT,             // anchor_height,
+    size_above_anchor,         // upper_size,
+    true,                      // negative,
+    false,                     // outline,
+    1.0,                       // line_width,
+    _padding,                  // padding_size,
     mask_color);
 
-  cairo_destroy(cr_bg);
-  cairo_destroy(cr_outline);
-  cairo_destroy(cr_mask);
-
   texture_bg_ = texture_ptr_from_cairo_graphics(cairo_bg);
   texture_mask_ = texture_ptr_from_cairo_graphics(cairo_mask);
   texture_outline_ = texture_ptr_from_cairo_graphics(cairo_outline);
@@ -1297,12 +1239,13 @@ std::string QuicklistView::GetName() const
   return "Quicklist";
 }
 
-void QuicklistView::AddProperties(GVariantBuilder* builder)
+void QuicklistView::AddProperties(debug::IntrospectionData& introspection)
 {
-  variant::BuilderWrapper(builder)
+  introspection
     .add(GetAbsoluteGeometry())
     .add("base_x", GetBaseX())
     .add("base_y", GetBaseY())
+    .add("base", nux::Point(GetBaseX(), GetBaseY()))
     .add("active", IsVisible());
 }
 
@@ -1326,12 +1269,12 @@ QuicklistView::GetSelectedMenuItem()
 
 debug::Introspectable::IntrospectableList QuicklistView::GetIntrospectableChildren()
 {
-  _introspectable_children.clear();
-  for (auto item: _item_list)
-  {
-    _introspectable_children.push_back(item);
-  }
-  return _introspectable_children;
+  debug::Introspectable::IntrospectableList list(_item_list.size());
+
+  for (auto const& item: _item_list)
+    list.push_back(item.GetPointer());
+
+  return list;
 }
 
 } // NAMESPACE
