@@ -24,6 +24,7 @@
 #include "unity-shared/AnimationUtils.h"
 #include "unity-shared/UBusMessages.h"
 #include "unity-shared/WindowManager.h"
+#include "unity-shared/IconRenderer.h"
 #include "unity-shared/UScreen.h"
 
 #include "SwitcherController.h"
@@ -44,7 +45,6 @@ const std::string DETAIL_TIMEOUT = "detail-timeout";
 const std::string VIEW_CONSTRUCT_IDLE = "view-construct-idle";
 const unsigned FADE_DURATION = 80;
 const int XY_OFFSET = 100;
-const int WH_OFFSET = -200;
 
 /**
  * Helper comparison functor for sorting application icons.
@@ -70,7 +70,9 @@ namespace switcher
 {
 
 Controller::Controller(WindowCreator const& create_window)
-  : detail_mode([this] { return detail_mode_; })
+  : detail([this] { return impl_->model_ && impl_->model_->detail_selection(); },
+           [this] (bool d) { if (impl_->model_) { impl_->model_->detail_selection = d; } return false; })
+  , detail_mode([this] { return detail_mode_; })
   , timeout_length(0)
   , detail_on_timeout(true)
   , detail_timeout_length(500)
@@ -78,6 +80,7 @@ Controller::Controller(WindowCreator const& create_window)
   , visible_(false)
   , monitor_(0)
   , show_desktop_disabled_(false)
+  , mouse_disabled_(false)
   , detail_mode_(DetailMode::TAB_NEXT_WINDOW)
   , impl_(new Controller::Impl(this, 20, create_window))
 {}
@@ -133,8 +136,7 @@ void Controller::Impl::StartDetailMode()
 {
   if (obj_->visible_)
   {
-    if (IsDetailViewShown() &&
-        HasNextDetailRow())
+    if (obj_->detail() && HasNextDetailRow())
     {
       NextDetailRow();
     }
@@ -149,8 +151,7 @@ void Controller::Impl::StopDetailMode()
 {
   if (obj_->visible_)
   {
-    if (IsDetailViewShown() &&
-        HasPrevDetailRow())
+    if (obj_->detail() && HasPrevDetailRow())
     {
       PrevDetailRow();
     }
@@ -176,11 +177,6 @@ SwitcherView::Ptr Controller::GetView() const
   return impl_->GetView();
 }
 
-bool Controller::IsDetailViewShown()
-{
-  return impl_->IsDetailViewShown();
-}
-
 void Controller::SetDetail(bool value, unsigned int min_windows)
 {
   impl_->SetDetail(value, min_windows);
@@ -201,16 +197,11 @@ void Controller::PrevDetail()
   impl_->PrevDetail();
 }
 
-LayoutWindow::Vector Controller::ExternalRenderTargets()
+LayoutWindow::Vector const& Controller::ExternalRenderTargets() const
 {
   return impl_->ExternalRenderTargets();
 }
 
-guint Controller::GetSwitcherInputWindowId() const
-{
-  return impl_->GetSwitcherInputWindowId();
-}
-
 bool Controller::IsShowDesktopDisabled() const
 {
   return show_desktop_disabled_;
@@ -251,11 +242,6 @@ sigc::connection Controller::ConnectToViewBuilt(const sigc::slot<void> &f)
   return impl_->view_built.connect(f);
 }
 
-void Controller::SetDetailOnTimeout(bool timeout)
-{
-  detail_on_timeout = timeout;
-}
-
 double Controller::Opacity() const
 {
   if (!impl_->view_window_)
@@ -271,9 +257,9 @@ Controller::GetName() const
 }
 
 void
-Controller::AddProperties(GVariantBuilder* builder)
+Controller::AddProperties(debug::IntrospectionData& introspection)
 {
-  unity::variant::BuilderWrapper(builder)
+  introspection
   .add("detail_on_timeout", detail_on_timeout())
   .add("initial_detail_timeout_length", initial_detail_timeout_length())
   .add("detail_timeout_length", detail_timeout_length())
@@ -290,6 +276,7 @@ Controller::Impl::Impl(Controller* obj,
   :  construct_timeout_(load_timeout)
   ,  obj_(obj)
   ,  create_window_(create_window)
+  ,  icon_renderer_(std::make_shared<ui::IconRenderer>())
   ,  main_layout_(nullptr)
   ,  fade_animator_(FADE_DURATION)
 {
@@ -301,7 +288,7 @@ Controller::Impl::Impl(Controller* obj,
     };
 
   // TODO We need to get actual timing data to suggest this is necessary.
-  //sources_.AddTimeoutSeconds(construct_timeout_, [&] { ConstructWindow(); return false; }, LAZY_TIMEOUT);
+  //sources_.AddTimeoutSeconds(construct_timeout_, [this] { ConstructWindow(); return false; }, LAZY_TIMEOUT);
 
   fade_animator_.updated.connect([this] (double opacity) {
     if (view_window_)
@@ -345,8 +332,8 @@ void Controller::Impl::Show(ShowMode show, SortMode sort, std::vector<AbstractLa
 
   if (real_wait > 0)
   {
-    sources_.AddIdle([&] { ConstructView(); return false; }, VIEW_CONSTRUCT_IDLE);
-    sources_.AddTimeout(real_wait, [&] { ShowView(); return false; }, SHOW_TIMEOUT);
+    sources_.AddIdle([this] { ConstructView(); return false; }, VIEW_CONSTRUCT_IDLE);
+    sources_.AddTimeout(real_wait, [this] { ShowView(); return false; }, SHOW_TIMEOUT);
   }
   else
   {
@@ -395,7 +382,7 @@ void Controller::Impl::OnModelSelectionChanged(AbstractLauncherIcon::Ptr const&
     }
 
     ubus_manager_.SendMessage(UBUS_SWITCHER_SELECTION_CHANGED,
-                              g_variant_new_string(icon->tooltip_text().c_str()));
+                              glib::Variant(icon->tooltip_text()));
   }
 }
 
@@ -419,7 +406,7 @@ void Controller::Impl::ShowView()
 
 void Controller::Impl::ConstructWindow()
 {
-  sources_.Remove(LAZY_TIMEOUT);
+  // sources_.Remove(LAZY_TIMEOUT);
 
   if (!view_window_)
   {
@@ -440,8 +427,7 @@ nux::Geometry GetSwitcherViewsGeometry()
   int monitor      = uscreen->GetMonitorWithMouse();
   auto monitor_geo = uscreen->GetMonitorGeometry(monitor);
 
-  monitor_geo.OffsetPosition(XY_OFFSET, XY_OFFSET);
-  monitor_geo.OffsetSize(WH_OFFSET, WH_OFFSET);
+  monitor_geo.Expand(-XY_OFFSET, -XY_OFFSET);
 
   return monitor_geo;
 }
@@ -453,7 +439,7 @@ void Controller::Impl::ConstructView()
 
   sources_.Remove(VIEW_CONSTRUCT_IDLE);
 
-  view_ = SwitcherView::Ptr(new SwitcherView());
+  view_ = SwitcherView::Ptr(new SwitcherView(icon_renderer_));
   obj_->AddChild(view_.GetPointer());
   view_->SetModel(model_);
   view_->background_color = WindowManager::Default().average_color();
@@ -462,13 +448,13 @@ void Controller::Impl::ConstructView()
   view_->hide_request.connect(sigc::mem_fun(this, &Controller::Impl::Hide));
 
   view_->switcher_mouse_up.connect([this] (int icon_index, int button) {
-      if (button == 3)
-        InitiateDetail(true);
+    if (button == 3)
+      InitiateDetail(true);
   });
 
   view_->switcher_mouse_move.connect([this] (int icon_index) {
-      if (icon_index >= 0)
-        ResetDetailTimer(obj_->detail_timeout_length);
+    if (icon_index >= 0)
+      ResetDetailTimer(obj_->detail_timeout_length);
   });
 
   view_->switcher_next.connect(sigc::mem_fun(this, &Impl::Next));
@@ -497,7 +483,7 @@ void Controller::Impl::Hide(bool accept_state)
     }
   }
 
-  ubus_manager_.SendMessage(UBUS_SWITCHER_END, g_variant_new_boolean(!accept_state));
+  ubus_manager_.SendMessage(UBUS_SWITCHER_END, glib::Variant(!accept_state));
   ubus_manager_.SendMessage(UBUS_SWITCHER_SHOWN, g_variant_new("(bi)", false, obj_->monitor_));
 
   sources_.Remove(VIEW_CONSTRUCT_IDLE);
@@ -513,12 +499,16 @@ void Controller::Impl::DetailHide()
 {
   // FIXME We need to refactor SwitcherModel so we can add/remove icons without causing
   // a crash. If you remove the last application in the list it crashes.
+  obj_->detail.changed.emit(false);
   model_->detail_selection = false;
   Hide(false);
 }
 
 void Controller::Impl::HideWindow()
 {
+  if (model_->detail_selection())
+    obj_->detail.changed.emit(false);
+
   main_layout_->RemoveChildObject(view_.GetPointer());
 
   view_window_->SetOpacity(0.0f);
@@ -592,20 +582,17 @@ SwitcherView::Ptr Controller::Impl::GetView() const
   return view_;
 }
 
-bool Controller::Impl::IsDetailViewShown()
-{
-  return model_ && model_->detail_selection();
-}
-
 void Controller::Impl::SetDetail(bool value, unsigned int min_windows)
 {
   if (value && model_->Selection()->AllowDetailViewInSwitcher() && model_->DetailXids().size() >= min_windows)
   {
     model_->detail_selection = true;
     obj_->detail_mode_ = DetailMode::TAB_NEXT_WINDOW;
+    obj_->detail.changed.emit(true);
   }
   else
   {
+    obj_->detail.changed.emit(false);
     model_->detail_selection = false;
   }
 }
@@ -617,20 +604,10 @@ void Controller::Impl::InitiateDetail(bool animate)
 
   if (!model_->detail_selection)
   {
-    view_->animate = animate;
-
     SetDetail(true);
 
-    if (!view_->animate())
-    {
-      // As soon as the detail selection is changed we re-enable the animations
-      auto conn = std::make_shared<sigc::connection>();
-      *conn = model_->detail_selection.changed.connect([this, conn] (bool) {
-        if (view_)
-          view_->animate = true;
-        conn->disconnect();
-      });
-    }
+    if (!animate)
+      view_->SkipAnimation();
   }
 }
 
@@ -684,22 +661,17 @@ bool Controller::Impl::HasPrevDetailRow() const
   return model_->HasPrevDetailRow();
 }
 
-LayoutWindow::Vector Controller::Impl::ExternalRenderTargets()
+LayoutWindow::Vector const& Controller::Impl::ExternalRenderTargets() const
 {
   if (!view_)
   {
-    LayoutWindow::Vector result;
-    return result;
+    static LayoutWindow::Vector empty_list;
+    return empty_list;
   }
+
   return view_->ExternalTargets();
 }
 
-guint Controller::Impl::GetSwitcherInputWindowId() const
-{
-  return view_window_->GetInputWindowId();
-}
-
-
 Selection Controller::Impl::GetCurrentSelection() const
 {
   AbstractLauncherIcon::Ptr application;
@@ -723,7 +695,6 @@ Selection Controller::Impl::GetCurrentSelection() const
   return {application, window};
 }
 
-
 void Controller::Impl::SelectFirstItem()
 {
   if (!model_)
@@ -754,7 +725,7 @@ void Controller::Impl::SelectFirstItem()
   for (auto& window : first->Windows())
   {
     Window xid = window->window_id();
-    
+
     if (model_->only_detail_on_viewport && !wm.IsWindowOnCurrentDesktop(xid))
       continue;
 
