@@ -35,7 +35,8 @@ unity::MT::X11TextureFactory::setActiveWrap (const GLTexture::List &t)
 unity::MT::Texture::Ptr
 unity::MT::X11TextureFactory::create ()
 {
-  return boost::shared_static_cast <unity::MT::Texture> (unity::MT::X11Texture::Ptr (new unity::MT::X11Texture (mWrap)));
+  unity::MT::Texture::Ptr tp(static_cast<unity::MT::Texture*> (new unity::MT::X11Texture(mWrap)));
+  return tp;
 }
 
 unity::MT::X11Texture::X11Texture (const GLTexture::List &t)
@@ -151,7 +152,7 @@ unity::MT::X11GrabHandleImpl::buttonPress (int x,
 }
 
 void
-UnityMTGrabHandlesWindow::raiseGrabHandle (const boost::shared_ptr <const unity::MT::GrabHandle> &h)
+UnityMTGrabHandlesWindow::raiseGrabHandle (const std::shared_ptr <const unity::MT::GrabHandle> &h)
 {
   UnityMTGrabHandlesScreen::get (screen)->raiseHandle (h, window->frame ());
 }
@@ -202,7 +203,7 @@ sortPointers(void *p1, void *p2)
 }
 
 void
-UnityMTGrabHandlesScreen::raiseHandle (const boost::shared_ptr <const unity::MT::GrabHandle> &h,
+UnityMTGrabHandlesScreen::raiseHandle (const std::shared_ptr <const unity::MT::GrabHandle> &h,
                                        Window                                                owner)
 {
   for (const auto &pair : mInputHandles)
@@ -372,7 +373,7 @@ UnityMTGrabHandlesScreen::donePaint()
     {
       if (handles->needsAnimate())
       {
-          handles->forEachHandle ([&](const unity::MT::GrabHandle::Ptr &h)
+          handles->forEachHandle ([this](const unity::MT::GrabHandle::Ptr &h)
 				  {
 				    h->damage (nux::Geometry (h->x (),
 							      h->y (),
@@ -421,7 +422,7 @@ UnityMTGrabHandlesWindow::allowHandles()
 void
 UnityMTGrabHandlesWindow::getOutputExtents(CompWindowExtents& output)
 {
-  auto f = [&] (const unity::MT::GrabHandle::Ptr &h)
+  auto f = [this, &output] (const unity::MT::GrabHandle::Ptr &h)
   {
     output.left = std::max (window->borderRect().left() + h->width () / 2, static_cast <unsigned int> (output.left));
     output.right = std::max (window->borderRect().right()  + h->width () / 2, static_cast <unsigned int> (output.right));
@@ -460,7 +461,7 @@ UnityMTGrabHandlesWindow::glDraw(const GLMatrix&            transform,
        * region */
       CompRegion reg = CompRegion(layout.second.x, layout.second.y, layout.second.width, layout.second.height);
 
-      for(GLTexture * tex : boost::shared_static_cast <unity::MT::X11Texture> (layout.first)->get ())
+      for(GLTexture * tex : static_cast<unity::MT::X11Texture*>(layout.first.get())->get())
       {
         GLTexture::MatrixList matl;
         GLTexture::Matrix     mat = tex->matrix();
@@ -629,7 +630,7 @@ UnityMTGrabHandlesWindow::restackHandles()
   if (!mHandles)
     return;
 
-  mHandles->forEachHandle ([&](const unity::MT::GrabHandle::Ptr &h)
+  mHandles->forEachHandle ([this](const unity::MT::GrabHandle::Ptr &h)
                            { h->reposition (0, 0, unity::MT::PositionLock); });
 }
 
@@ -777,7 +778,7 @@ UnityMTGrabHandlesScreen::UnityMTGrabHandlesScreen(CompScreen* s) :
     GLTexture::List t = GLTexture::readImageToTexture(fname, pname,
                                                       size);
 
-    (boost::shared_static_cast <unity::MT::X11TextureFactory> (unity::MT::Texture::Factory::Default ()))->setActiveWrap (t);
+    (static_cast<unity::MT::X11TextureFactory*>(unity::MT::Texture::Factory::Default().get())->setActiveWrap(t));
 
     mHandleTextures.at(i).first = unity::MT::Texture::Factory::Default ()->create ();
     mHandleTextures.at (i).second.width = size.width ();
