@@ -309,7 +309,7 @@ public:
   void    moveTransientsToRect(CompWindow* skip, CompRect rect, bool);
   void    moveParentToRect(CompWindow* requestor, CompRect rect, bool);
 
-  void      moveToRect(CompRect currentRect, bool sync);
+  void      moveToRect(CompRect currentRect);
 
   void    grabTransients(CompWindow* skip, int x, int y,
                          unsigned int state, unsigned int mask, bool);
@@ -321,7 +321,7 @@ public:
 
   void      setMaxConstrainingAreas();
 
-  CompWindow* transientParent();
+  CompWindow* transientParent() const;
 
   void adjustIPW();
 
