@@ -64,7 +64,7 @@ static gboolean   unity_switcher_accessible_is_child_selected(AtkSelection* sele
 static gboolean   unity_switcher_accessible_check_pending_notification(NuxAreaAccessible* self);
 
 /* private */
-static void       on_selection_changed_cb(AbstractLauncherIcon::Ptr icon,
+static void       on_selection_changed_cb(AbstractLauncherIcon::Ptr const& icon,
                                           UnitySwitcherAccessible* switcher_accessible);
 static void       create_children(UnitySwitcherAccessible* self);
 
@@ -311,7 +311,6 @@ unity_switcher_accessible_get_selection_count(AtkSelection* selection)
 {
   SwitcherView* switcher = NULL;
   SwitcherModel::Ptr switcher_model;
-  AbstractLauncherIcon::Ptr selected_icon;
   nux::Object* nux_object = NULL;
 
   g_return_val_if_fail(UNITY_IS_SWITCHER_ACCESSIBLE(selection), 0);
@@ -323,9 +322,7 @@ unity_switcher_accessible_get_selection_count(AtkSelection* selection)
   switcher = dynamic_cast<SwitcherView*>(nux_object);
   switcher_model = switcher->GetModel();
 
-  selected_icon = switcher_model->Selection();
-
-  if (!selected_icon)
+  if (!switcher_model->Selection())
     return 0;
   else
     return 1;
@@ -377,7 +374,7 @@ unity_switcher_accessible_check_pending_notification(NuxAreaAccessible* self)
 
 /* private */
 static void
-on_selection_changed_cb(AbstractLauncherIcon::Ptr icon,
+on_selection_changed_cb(AbstractLauncherIcon::Ptr const& icon,
                         UnitySwitcherAccessible* switcher_accessible)
 {
   g_signal_emit_by_name(ATK_OBJECT(switcher_accessible), "selection-changed");
@@ -389,9 +386,7 @@ create_children(UnitySwitcherAccessible* self)
   gint index = 0;
   nux::Object* nux_object = NULL;
   SwitcherView* switcher = NULL;
-  SwitcherModel::Ptr switcher_model;
   SwitcherModel::iterator it;
-  AbstractLauncherIcon::Ptr child;
   AtkObject* child_accessible = NULL;
 
   nux_object = nux_object_accessible_get_object(NUX_OBJECT_ACCESSIBLE(self));
@@ -399,14 +394,13 @@ create_children(UnitySwitcherAccessible* self)
     return;
 
   switcher = dynamic_cast<SwitcherView*>(nux_object);
-  switcher_model = switcher->GetModel();
+  SwitcherModel::Ptr const& switcher_model = switcher->GetModel();
 
-  if (switcher_model == NULL)
+  if (!switcher_model)
     return;
 
-  for (it = switcher_model->begin(); it != switcher_model->end(); ++it)
+  for (AbstractLauncherIcon::Ptr const& child : *switcher_model)
   {
-    child = *it;
     child_accessible = unity_launcher_icon_accessible_new(child.GetPointer());
     atk_object_set_parent(child_accessible, ATK_OBJECT(self));
     self->priv->children = g_slist_append(self->priv->children,
