@@ -101,8 +101,8 @@ void Controller::Show(View::Mode mode, bool inhibitors)
   view_->live_background = true;
 
   view_window_->ShowWindow(true);
-  view_window_->PushToFront();
   view_window_->SetInputFocus();
+  nux::GetWindowCompositor().SetAlwaysOnFrontWindow(view_window_.GetPointer());
   nux::GetWindowCompositor().SetKeyFocusArea(view_->key_focus_area());
   animation::StartOrReverse(fade_animator_, animation::Direction::FORWARD);
 }
@@ -151,6 +151,8 @@ void Controller::ConstructView()
   {
     view_->size_changed.connect([this] (nux::Area*, int, int) {
       int monitor = UScreen::GetDefault()->GetMonitorWithMouse();
+      view_->monitor = monitor;
+
       auto const& offset = GetOffsetPerMonitor(monitor);
       view_window_->SetXY(offset.x, offset.y);
     });
@@ -175,17 +177,19 @@ void Controller::CancelAndHide()
 
 void Controller::Hide()
 {
-  animation::StartOrReverse(fade_animator_, animation::Direction::BACKWARD);
+  if (view_window_)
+    animation::StartOrReverse(fade_animator_, animation::Direction::BACKWARD);
 }
 
 void Controller::CloseWindow()
 {
-  view_window_->PushToBack();
-  view_window_->ShowWindow(false);
   view_window_->UnGrabPointer();
   view_window_->UnGrabKeyboard();
+  view_window_->ShowWindow(false);
   view_window_->EnableInputWindow(false);
-  view_->live_background = false;
+
+  view_ = nullptr;
+  view_window_ = nullptr;
 
   nux::GetWindowCompositor().SetKeyFocusArea(nullptr);
   WindowManager::Default().RestoreInputFocus();
