@@ -190,15 +190,34 @@ void PluginAdapter::NotifyCompizEvent(const char* plugin,
     _vp_switch_started = false;
     screen_viewport_switch_ended.emit();
   }
-  else if (IsScaleActive() && g_strcmp0(plugin, "scale") == 0 &&
-           g_strcmp0(event, "activate") == 0)
+  else if (g_strcmp0(plugin, "scale") == 0 && g_strcmp0(event, "activate") == 0)
   {
-    // If the scale plugin is activated again while is already grabbing the screen
-    // it means that is switching the view (i.e. switching from a spread application
-    // to another), so we need to notify our clients that it has really terminated
-    // and initiated again.
-    terminate_spread.emit();
-    initiate_spread.emit();
+    bool new_state = CompOption::getBoolOptionNamed(option, "active");
+
+    if (_spread_state != new_state)
+    {
+      _spread_state = new_state;
+      _spread_state ? initiate_spread.emit() : terminate_spread.emit();
+
+      if (!_spread_state)
+        _spread_windows_state = false;
+    }
+    else if (_spread_state && new_state)
+    {
+      // If the scale plugin is activated again while is already grabbing the screen
+      // it means that is switching the view (i.e. switching from a spread application
+      // to another), so we need to notify our clients that it has really terminated
+      // and initiated again.
+
+      bool old_windows_state = _spread_windows_state;
+      _spread_state = false;
+      _spread_windows_state = false;
+      terminate_spread.emit();
+
+      _spread_state = true;
+      _spread_windows_state = old_windows_state;
+      initiate_spread.emit();
+    }
   }
 }
 
@@ -248,7 +267,8 @@ void MultiActionList::Initiate(std::string const& name, CompOption::Vector const
     argument.push_back(arg);
 
   /* Initiate the selected action with the arguments */
-  action->initiate()(action, state, argument);
+  if (CompAction::CallBack const& initiate_cb = primary_action_->initiate())
+    initiate_cb(action, state, argument);
 }
 
 void MultiActionList::InitiateAll(CompOption::Vector const& extra_args, int state) const
@@ -291,8 +311,11 @@ void MultiActionList::TerminateAll(CompOption::Vector const& extra_args) const
 
   if (primary_action_)
   {
-    primary_action_->terminate()(primary_action_, CompAction::StateCancel, argument);
-    return;
+    if (CompAction::CallBack const& terminate_cb = primary_action_->terminate())
+    {
+      terminate_cb(primary_action_, CompAction::StateCancel, argument);
+      return;
+    }
   }
 
   for (auto const& it : actions_)
@@ -304,7 +327,8 @@ void MultiActionList::TerminateAll(CompOption::Vector const& extra_args) const
                            CompAction::StateTermEdge |
                            CompAction::StateTermEdgeDnd))
     {
-      action->terminate()(action, 0, argument);
+      if (CompAction::CallBack const& terminate_cb = primary_action_->terminate())
+        terminate_cb(action, 0, argument);
     }
   }
 }
@@ -1221,7 +1245,21 @@ int PluginAdapter::GetViewportVSize() const
 
 bool PluginAdapter::IsScreenGrabbed() const
 {
-  return m_Screen->grabbed();
+  if (m_Screen->grabbed())
+    return true;
+
+  auto* dpy = m_Screen->dpy();
+  auto ret = XGrabKeyboard(dpy, m_Screen->root(), True, GrabModeAsync, GrabModeAsync, CurrentTime);
+
+  if (ret == GrabSuccess)
+  {
+    XUngrabKeyboard(dpy, CurrentTime);
+
+    if (CompWindow* w = m_Screen->findWindow(m_Screen->activeWindow()))
+      w->moveInputFocusTo();
+  }
+
+  return (ret == AlreadyGrabbed);
 }
 
 bool PluginAdapter::IsViewPortSwitchStarted() const
