@@ -35,6 +35,7 @@ namespace dash
 {
 namespace previews
 {
+DECLARE_LOGGER(logger, "unity.dash.previews.style");
 namespace
 {
 Style* style_instance = nullptr;
@@ -42,8 +43,6 @@ Style* style_instance = nullptr;
 const int preview_width = 770;
 const int preview_height = 380;
 
-nux::logging::Logger logger("unity.dash.previews.style");
-
 typedef nux::ObjectPtr<nux::BaseTexture> BaseTexturePtr;
 
 template <int default_size = 1>
@@ -100,6 +99,7 @@ public:
   , preview_play_texture_("/preview_play.svg")
   , preview_pause_texture_("/preview_pause.svg")
   , preview_spin_texture_("/search_spin.svg")
+  , warning_icon_texture_("/warning_icon.png")
   {
   }
   ~Impl() {}
@@ -111,6 +111,7 @@ public:
   LazyLoadTexture<32> preview_play_texture_;
   LazyLoadTexture<32> preview_pause_texture_;
   LazyLoadTexture<32> preview_spin_texture_;
+  LazyLoadTexture<22> warning_icon_texture_;
 };
 
 
@@ -172,6 +173,12 @@ int Style::GetPreviewHeight() const
   return preview_height;
 }
 
+
+int Style::GetPreviewTopPadding() const
+{
+  return 100;
+}
+
 int Style::GetDetailsTopMargin() const
 {
   return 5;
@@ -287,6 +294,96 @@ int Style::GetStatusIconSize() const
   return 12;
 }
 
+std::string Style::payment_title_font() const
+{
+  return "Ubuntu 22";
+}
+
+std::string Style::payment_subtitle_font() const
+{
+  return "Ubuntu 12.5";
+}
+
+std::string Style::payment_prize_title_font() const
+{
+  return "Ubuntu Bold 12.5";
+}
+
+std::string Style::payment_prize_subtitle_font() const
+{
+  return "Ubuntu 10";
+}
+
+std::string Style::payment_intro_font() const
+{
+  return "Ubuntu 11.7";
+}
+
+std::string Style::payment_form_labels_font() const
+{
+  return "Ubuntu 10";
+}
+
+std::string Style::payment_form_data_font() const
+{
+  return "Ubuntu Bold 10";
+}
+
+std::string Style::payment_form_actions_font() const
+{
+  return "Ubuntu 10";
+}
+
+std::string Style::payment_text_input_font() const
+{
+  return "Ubuntu 14";
+}
+
+nux::Color Style::payment_error_color() const
+{
+  return nux::Color(255, 0, 0);
+}
+
+int Style::GetPaymentIconAreaWidth() const
+{
+  return 64;
+}
+
+int Style::GetPaymentTextInputHeight() const
+{
+  return 40;
+}
+
+int Style::GetPaymentLockWidth() const
+{
+  return 22;
+}
+
+int Style::GetPaymentLockHeight() const
+{
+  return 22;
+}
+
+int Style::GetPaymentHeaderWidth() const
+{
+  return 850;
+}
+
+int Style::GetPaymentHeaderSpace() const
+{
+  return 0;
+}
+
+int Style::GetPaymentFormSpace() const
+{
+  return 5;
+}
+
+std::string Style::u1_warning_font() const
+{
+  return "Ubuntu Bold 11.5";
+}
+
 float Style::GetVideoImageAspectRatio() const
 {
   return float(540)/380;
@@ -396,6 +493,17 @@ nux::BaseTexture* Style::GetPauseIcon()
   return pimpl->preview_pause_texture_.texture();
 }
 
+nux::BaseTexture* Style::GetLockIcon()
+{
+  return nux::CreateTexture2DFromFile(
+              PKGDATADIR"/lock_icon.png", -1, true);
+}
+
+nux::BaseTexture* Style::GetWarningIcon()
+{
+  return pimpl->warning_icon_texture_.texture();
+}
+
 nux::BaseTexture* Style::GetSearchSpinIcon(int size)
 {
   return pimpl->preview_spin_texture_.texture(size);
