@@ -1,6 +1,6 @@
 // -*- Mode: C++; indent-tabs-mode: nil; tab-width: 2 -*-
 /*
- * Copyright (C) 2010-2011 Canonical Ltd
+ * Copyright (C) 2010-2014 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -24,27 +24,42 @@
 #include <Nux/VLayout.h>
 #include <NuxCore/Logger.h>
 
-#include <UnityCore/Variant.h>
-
 #include <glib/gi18n-lib.h>
 
 #include "SearchBar.h"
 #include "CairoTexture.h"
 #include "DashStyle.h"
 #include "GraphicsUtils.h"
+#include "RawPixel.h"
+#include "UnitySettings.h"
+
+namespace unity
+{
 
 namespace
 {
-const float kExpandDefaultIconOpacity = 1.0f;
-const int LIVE_SEARCH_TIMEOUT = 40;
+const double DEFAULT_SCALE = 1.0;
+const float DEFAULT_ICON_OPACITY = 1.0f;
+const int DEFAULT_LIVE_SEARCH_TIMEOUT = 40;
 const int SPINNER_TIMEOUT = 100;
 
-const int SPACE_BETWEEN_SPINNER_AND_TEXT = 5;
-const int SPACE_BETWEEN_ENTRY_AND_HIGHLIGHT = 10;
-const int LEFT_INTERNAL_PADDING = 4;
-const int SEARCH_ENTRY_RIGHT_BORDER = 10;
+const RawPixel SPACE_BETWEEN_SPINNER_AND_TEXT    =  5_em;
+const RawPixel SPACE_BETWEEN_ENTRY_AND_HIGHLIGHT = 10_em;
+const RawPixel LEFT_INTERNAL_PADDING     =  4_em;
+const RawPixel SEARCH_ENTRY_RIGHT_BORDER = 10_em;
+const RawPixel LEFT_PADDING  =  0_em;
+const RawPixel RIGHT_PADDING = 10_em;
 
-const int HIGHLIGHT_HEIGHT = 24;
+const RawPixel HIGHLIGHT_HEIGHT = 24_em;
+
+const RawPixel ARROW_MIN_WIDTH = 2_em;
+const RawPixel ARROW_MAX_WIDTH = 2_em;
+
+const RawPixel TOP_ARROW_MIN_HEIGHT = 12_em;
+const RawPixel TOP_ARROW_MAX_HEIGHT = 12_em;
+
+const RawPixel BOT_ARROW_MIN_HEIGHT = 8_em;
+const RawPixel BOT_ARROW_MAX_HEIGHT = 8_em;
 
 // Fonts
 const std::string HINT_LABEL_FONT_SIZE = "20px";
@@ -52,7 +67,7 @@ const std::string HINT_LABEL_FONT_STYLE = "Italic";
 const std::string HINT_LABEL_DEFAULT_FONT = "Ubuntu " + HINT_LABEL_FONT_STYLE + " " + HINT_LABEL_FONT_SIZE;
 
 const std::string PANGO_ENTRY_DEFAULT_FONT_FAMILY = "Ubuntu";
-const int PANGO_ENTRY_FONT_SIZE = 22;
+const RawPixel PANGO_ENTRY_FONT_SIZE = 22_em;
 
 const std::string SHOW_FILTERS_LABEL_FONT_SIZE = "13";
 const std::string SHOW_FILTERS_LABEL_FONT_STYLE = "";
@@ -60,6 +75,8 @@ const std::string SHOW_FILTERS_LABEL_DEFAULT_FONT = "Ubuntu " + SHOW_FILTERS_LAB
 
 }
 
+}
+
 DECLARE_LOGGER(logger, "unity.dash.searchbar");
 namespace
 {
@@ -107,54 +124,34 @@ namespace unity
 NUX_IMPLEMENT_OBJECT_TYPE(SearchBar);
 
 SearchBar::SearchBar(NUX_FILE_LINE_DECL)
+  : SearchBar(false)
+{}
+
+SearchBar::SearchBar(bool show_filter_hint, NUX_FILE_LINE_DECL)
   : View(NUX_FILE_LINE_PARAM)
-  , search_hint("")
   , showing_filters(false)
   , can_refine_search(false)
-  , show_filter_hint_(true)
+  , live_search_wait(DEFAULT_LIVE_SEARCH_TIMEOUT)
+  , scale(DEFAULT_SCALE)
+  , show_filter_hint_(show_filter_hint)
   , expander_view_(nullptr)
   , show_filters_(nullptr)
   , last_width_(-1)
   , last_height_(-1)
-{
-  Init();
-}
-
-SearchBar::SearchBar(bool show_filter_hint_, NUX_FILE_LINE_DECL)
-  : View(NUX_FILE_LINE_PARAM)
-  , search_hint("")
-  , showing_filters(false)
-  , can_refine_search(false)
-  , show_filter_hint_(show_filter_hint_)
-  , expander_view_(nullptr)
-  , show_filters_(nullptr)
-  , last_width_(-1)
-  , last_height_(-1)
-{
-  Init();
-}
-
-void SearchBar::Init()
 {
   dash::Style& style = dash::Style::Instance();
-  nux::BaseTexture* icon = style.GetSearchMagnifyIcon();
 
   bg_layer_.reset(new nux::ColorLayer(nux::Color(0xff595853), true));
 
   layout_ = new nux::HLayout(NUX_TRACKER_LOCATION);
-  layout_->SetLeftAndRightPadding(LEFT_INTERNAL_PADDING, SEARCH_ENTRY_RIGHT_BORDER);
-  layout_->SetSpaceBetweenChildren(SPACE_BETWEEN_ENTRY_AND_HIGHLIGHT);
   SetLayout(layout_);
 
   entry_layout_ = new nux::HLayout(NUX_TRACKER_LOCATION);
-  entry_layout_->SetLeftAndRightPadding(0, 10);
   layout_->AddLayout(entry_layout_);
 
   spinner_ = new SearchBarSpinner();
-  spinner_->SetMinMaxSize(icon->GetWidth(), icon->GetHeight());
   spinner_->mouse_click.connect(sigc::mem_fun(this, &SearchBar::OnClearClicked));
   entry_layout_->AddView(spinner_, 0, nux::MINOR_POSITION_CENTER, nux::MINOR_SIZE_FULL);
-  entry_layout_->SetSpaceBetweenChildren(SPACE_BETWEEN_SPINNER_AND_TEXT);
 
   nux::HLayout* hint_layout = new nux::HLayout(NUX_TRACKER_LOCATION);
 
@@ -165,10 +162,9 @@ void SearchBar::Init()
 
   pango_entry_ = new IMTextEntry();
   pango_entry_->SetFontFamily(PANGO_ENTRY_DEFAULT_FONT_FAMILY.c_str());
-  pango_entry_->SetFontSize(PANGO_ENTRY_FONT_SIZE);
   pango_entry_->text_changed.connect(sigc::mem_fun(this, &SearchBar::OnSearchChanged));
-  pango_entry_->activated.connect([&]() { activated.emit(); });
-  pango_entry_->cursor_moved.connect([&](int i) { QueueDraw(); });
+  pango_entry_->activated.connect([this]() { activated.emit(); });
+  pango_entry_->cursor_moved.connect([this](int i) { QueueDraw(); });
   pango_entry_->mouse_down.connect(sigc::mem_fun(this, &SearchBar::OnMouseButtonDown));
   pango_entry_->end_key_focus.connect(sigc::mem_fun(this, &SearchBar::OnEndKeyFocus));
   pango_entry_->key_up.connect([this] (unsigned int, unsigned long, unsigned long) {
@@ -201,19 +197,28 @@ void SearchBar::Init()
     expand_icon_ = new IconTexture(arrow,
                                    arrow->GetWidth(),
                                    arrow->GetHeight());
-    expand_icon_->SetOpacity(kExpandDefaultIconOpacity);
+    expand_icon_->SetOpacity(DEFAULT_ICON_OPACITY);
     expand_icon_->SetMinimumSize(arrow->GetWidth(), arrow->GetHeight());
     expand_icon_->SetVisible(false);
 
     filter_layout_ = new nux::HLayout();
     filter_layout_->SetHorizontalInternalMargin(8);
-    filter_layout_->SetLeftAndRightPadding(style.GetFilterResultsHighlightLeftPadding(), style.GetFilterResultsHighlightRightPadding());
+    filter_layout_->SetLeftAndRightPadding(style.GetFilterResultsHighlightLeftPadding(),
+                                           style.GetFilterResultsHighlightRightPadding());
     filter_layout_->SetContentDistribution(nux::MAJOR_POSITION_END);
     filter_layout_->AddView(show_filters_, 0, nux::MINOR_POSITION_CENTER);
 
     arrow_layout_  = new nux::VLayout();
-    arrow_top_space_ = new nux::SpaceLayout(2, 2, 12, 12);
-    arrow_bottom_space_ = new nux::SpaceLayout(2, 2, 8, 8);
+    arrow_top_space_ = new nux::SpaceLayout(ARROW_MIN_WIDTH.CP(scale()),
+                                            ARROW_MAX_WIDTH.CP(scale()),
+                                            TOP_ARROW_MIN_HEIGHT.CP(scale()),
+                                            TOP_ARROW_MAX_HEIGHT.CP(scale()));
+
+    arrow_bottom_space_ = new nux::SpaceLayout(ARROW_MIN_WIDTH.CP(scale()),
+                                               ARROW_MAX_WIDTH.CP(scale()),
+                                               BOT_ARROW_MIN_HEIGHT.CP(scale()),
+                                               BOT_ARROW_MAX_HEIGHT.CP(scale()));
+
     arrow_layout_->AddView(arrow_top_space_, 0, nux::MINOR_POSITION_CENTER);
     arrow_layout_->AddView(expand_icon_, 0, nux::MINOR_POSITION_CENTER);
     arrow_layout_->AddView(arrow_bottom_space_, 0, nux::MINOR_POSITION_CENTER);
@@ -234,17 +239,17 @@ void SearchBar::Init()
     show_filters_->SetMaximumWidth(style.GetFilterBarWidth() - arrow_layout_->GetBaseWidth() - 8);
 
     // Lambda functions
-    auto mouse_expand = [&](int, int, unsigned long, unsigned long)
+    auto mouse_expand = [this](int, int, unsigned long, unsigned long)
     {
       showing_filters = !showing_filters;
     };
 
-    auto key_redraw = [&](nux::Area*, bool, nux::KeyNavDirection)
+    auto key_redraw = [this](nux::Area*, bool, nux::KeyNavDirection)
     {
       QueueDraw();
     };
 
-    auto key_expand = [&](nux::Area*)
+    auto key_expand = [this](nux::Area*)
     {
       showing_filters = !showing_filters;
     };
@@ -261,13 +266,14 @@ void SearchBar::Init()
                                                     sigc::mem_fun(this, &SearchBar::OnFontChanged));
   OnFontChanged(gtk_settings_get_default());
 
-  search_hint.changed.connect([&](std::string const& s) { OnSearchHintChanged(); });
+  search_hint.changed.connect([this](std::string const& s) { OnSearchHintChanged(); });
   search_string.SetGetterFunction(sigc::mem_fun(this, &SearchBar::get_search_string));
   search_string.SetSetterFunction(sigc::mem_fun(this, &SearchBar::set_search_string));
   im_active.SetGetterFunction(sigc::mem_fun(this, &SearchBar::get_im_active));
   im_preedit.SetGetterFunction(sigc::mem_fun(this, &SearchBar::get_im_preedit));
   showing_filters.changed.connect(sigc::mem_fun(this, &SearchBar::OnShowingFiltersChanged));
-  can_refine_search.changed.connect([&] (bool can_refine)
+  scale.changed.connect(sigc::mem_fun(this, &SearchBar::UpdateScale));
+  can_refine_search.changed.connect([this] (bool can_refine)
   {
     if (show_filter_hint_)
     {
@@ -276,6 +282,61 @@ void SearchBar::Init()
       expand_icon_->SetVisible(can_refine);
     }
   });
+
+  UpdateSearchBarSize();
+}
+
+void SearchBar::UpdateSearchBarSize()
+{
+  dash::Style& style = dash::Style::Instance();
+  nux::BaseTexture* icon = style.GetSearchMagnifyIcon();
+  RawPixel const icon_width  = icon->GetWidth();
+  RawPixel const icon_height = icon->GetHeight();
+
+  layout_->SetLeftAndRightPadding(LEFT_INTERNAL_PADDING.CP(scale()),
+                                  SEARCH_ENTRY_RIGHT_BORDER.CP(scale()));
+  layout_->SetSpaceBetweenChildren(SPACE_BETWEEN_ENTRY_AND_HIGHLIGHT.CP(scale()));
+
+  entry_layout_->SetLeftAndRightPadding(LEFT_PADDING.CP(scale()),
+                                        RIGHT_PADDING.CP(scale()));
+
+  spinner_->SetMinMaxSize(icon_width.CP(scale()), icon_height.CP(scale()));
+  entry_layout_->SetSpaceBetweenChildren(SPACE_BETWEEN_SPINNER_AND_TEXT.CP(scale()));
+
+  pango_entry_->SetFontSize(PANGO_ENTRY_FONT_SIZE.CP(scale()));
+
+  if (show_filter_hint_)
+  {
+    arrow_top_space_->SetMinimumSize(ARROW_MIN_WIDTH.CP(scale()),
+                                     TOP_ARROW_MIN_HEIGHT.CP(scale()));
+    arrow_top_space_->SetMaximumSize(ARROW_MAX_WIDTH.CP(scale()),
+                                     TOP_ARROW_MAX_HEIGHT.CP(scale()));
+
+    arrow_bottom_space_->SetMinimumSize(ARROW_MIN_WIDTH.CP(scale()),
+                                        BOT_ARROW_MIN_HEIGHT.CP(scale()));
+    arrow_bottom_space_->SetMaximumSize(ARROW_MAX_WIDTH.CP(scale()),
+                                        BOT_ARROW_MAX_HEIGHT.CP(scale()));
+  }
+
+  // Based on the Font size, the MinHeight is changing in TextEntry. From there the
+  // layered_layout grows to match the MinHeight, but when the MinHeight is shurnk it
+  // is not changing (since the MaxHeight is int::MAX). Now we grab the MinHeight from 
+  // PangoEntry, and set it for layered_layout.
+  int entry_min = pango_entry_->GetMinimumHeight();
+
+  pango_entry_->SetMaximumHeight(entry_min);
+  layered_layout_->SetMinimumHeight(entry_min);
+  layered_layout_->SetMaximumHeight(entry_min);
+}
+
+void SearchBar::UpdateScale(double scale)
+{
+  hint_->SetScale(scale);
+
+  if (show_filters_)
+    show_filters_->SetScale(scale);
+
+  UpdateSearchBarSize();
 }
 
 void SearchBar::OnFontChanged(GtkSettings* settings, GParamSpec* pspec)
@@ -296,10 +357,13 @@ void SearchBar::OnFontChanged(GtkSettings* settings, GParamSpec* pspec)
     font_desc << pango_font_description_get_family(desc) << " " << HINT_LABEL_FONT_STYLE << " " << HINT_LABEL_FONT_SIZE;
     hint_->SetFont(font_desc.str().c_str());
 
-    font_desc.str("");
-    font_desc.clear();
-    font_desc << pango_font_description_get_family(desc) << " " << SHOW_FILTERS_LABEL_FONT_STYLE << " " << SHOW_FILTERS_LABEL_FONT_SIZE;
-    show_filters_->SetFont(font_desc.str().c_str());
+    if (show_filter_hint_)
+    {
+      font_desc.str("");
+      font_desc.clear();
+      font_desc << pango_font_description_get_family(desc) << " " << SHOW_FILTERS_LABEL_FONT_STYLE << " " << SHOW_FILTERS_LABEL_FONT_SIZE;
+      show_filters_->SetFont(font_desc.str().c_str());
+    }
 
     pango_font_description_free(desc);
   }
@@ -319,7 +383,7 @@ void SearchBar::OnSearchChanged(nux::TextEntry* text_entry)
   // We don't want to set a new search string on every new character, so we add a sma
   // timeout to see if the user is typing a sentence. If more characters are added, we
   // keep restarting the timeout unti the user has actuay paused.
-  live_search_timeout_.reset(new glib::Timeout(LIVE_SEARCH_TIMEOUT));
+  live_search_timeout_.reset(new glib::Timeout(live_search_wait()));
   live_search_timeout_->Run(sigc::mem_fun(this, &SearchBar::OnLiveSearchTimeout));
 
   // Don't animate the spinner immediately, the searches are fast and
@@ -384,8 +448,8 @@ void SearchBar::Draw(nux::GraphicsEngine& graphics_engine, bool force_draw)
 
     nux::Geometry geo(expander_view_->GetGeometry());
 
-    geo.y -= (HIGHLIGHT_HEIGHT- geo.height) / 2;
-    geo.height = HIGHLIGHT_HEIGHT;
+    geo.y -= (HIGHLIGHT_HEIGHT.CP(scale()) - geo.height) / 2;
+    geo.height = HIGHLIGHT_HEIGHT.CP(scale());
 
     if (!highlight_layer_)
       highlight_layer_.reset(style.FocusOverlay(geo.width, geo.height));
@@ -430,6 +494,9 @@ void SearchBar::DrawContent(nux::GraphicsEngine& graphics_engine, bool force_dra
     nux::GetPainter().PushPaintLayerStack();      
   }
 
+  if (!IsFullRedraw())
+    graphics::ClearGeometry(pango_entry_->GetGeometry());
+
   layout_->ProcessDraw(graphics_engine, force_draw);
 
   if (IsFullRedraw())
@@ -457,7 +524,7 @@ void SearchBar::OnEntryActivated()
 
 void SearchBar::ForceLiveSearch()
 {
-  live_search_timeout_.reset(new glib::Timeout(LIVE_SEARCH_TIMEOUT));
+  live_search_timeout_.reset(new glib::Timeout(live_search_wait()));
   live_search_timeout_->Run(sigc::mem_fun(this, &SearchBar::OnLiveSearchTimeout));
 
   start_spinner_timeout_.reset(new glib::Timeout(SPINNER_TIMEOUT));
@@ -468,19 +535,18 @@ void SearchBar::SetSearchFinished()
 {
   start_spinner_timeout_.reset();
 
-  bool is_empty = pango_entry_->im_active() ?
-    false : pango_entry_->GetText() == "";
+  bool is_empty = pango_entry_->im_active() ? false : pango_entry_->GetText().empty();
   spinner_->SetState(is_empty ? STATE_READY : STATE_CLEAR);
 }
 
 void SearchBar::UpdateBackground(bool force)
 {
-  int RADIUS = 5;
+  RawPixel const RADIUS = 5;
   nux::Geometry geo(GetGeometry());
   geo.width = layered_layout_->GetAbsoluteX() +
               layered_layout_->GetAbsoluteWidth() -
               GetAbsoluteX() +
-              SEARCH_ENTRY_RIGHT_BORDER;
+              SEARCH_ENTRY_RIGHT_BORDER.CP(scale());
 
   LOG_TRACE(logger) << "height: "
   << geo.height << " - "
@@ -497,12 +563,12 @@ void SearchBar::UpdateBackground(bool force)
   last_height_ = geo.height;
 
   nux::CairoGraphics cairo_graphics(CAIRO_FORMAT_ARGB32, last_width_, last_height_);
-  cairo_t* cr = cairo_graphics.GetContext();
+  cairo_t* cr = cairo_graphics.GetInternalContext();
 
   cairo_graphics.DrawRoundedRectangle(cr,
                                       1.0f,
                                       0.5, 0.5,
-                                      RADIUS,
+                                      RADIUS.CP(scale()),
                                       last_width_ - 1, last_height_ - 1,
                                       false);
 
@@ -513,7 +579,6 @@ void SearchBar::UpdateBackground(bool force)
   cairo_set_source_rgba(cr, 1.0f, 1.0f, 1.0f, 0.7f);
   cairo_stroke(cr);
 
-  cairo_destroy(cr);
   nux::BaseTexture* texture2D = texture_from_cairo_graphics(cairo_graphics);
 
   nux::TexCoordXForm texxform;
@@ -605,19 +670,25 @@ std::string SearchBar::GetName() const
   return "SearchBar";
 }
 
-void SearchBar::AddProperties(GVariantBuilder* builder)
+void SearchBar::AddProperties(debug::IntrospectionData& introspection)
 {
-  unity::variant::BuilderWrapper(builder)
+  introspection
   .add(GetAbsoluteGeometry())
   .add("has_focus", pango_entry_->HasKeyFocus())
   .add("search_string", pango_entry_->GetText())
-  .add("expander-has-focus", expander_view_->HasKeyFocus())
   .add("showing-filters", showing_filters)
-  .add("filter-label-x", show_filters_->GetAbsoluteX())
-  .add("filter-label-y", show_filters_->GetAbsoluteY())
-  .add("filter-label-width", show_filters_->GetAbsoluteWidth())
-  .add("filter-label-height", show_filters_->GetAbsoluteHeight())
   .add("im_active", pango_entry_->im_active());
+
+  if (show_filter_hint_)
+  {
+    introspection
+    .add("expander-has-focus", expander_view_->HasKeyFocus())
+    .add("filter-label-x", show_filters_->GetAbsoluteX())
+    .add("filter-label-y", show_filters_->GetAbsoluteY())
+    .add("filter-label-width", show_filters_->GetAbsoluteWidth())
+    .add("filter-label-height", show_filters_->GetAbsoluteHeight())
+    .add("filter-label-geo", show_filters_->GetAbsoluteGeometry());
+  }
 }
 
 } // namespace unity
