@@ -62,6 +62,8 @@ public:
   void SetSpinnerVisible(bool visible);
   void SetSpinnerState(SpinnerState spinner_state);
 
+  void EnableCapsLockWarning() const;
+
   IMTextEntry* text_entry() const;
 
   nux::RWProperty<std::string> input_string;
@@ -70,6 +72,7 @@ public:
   nux::Property<int> hint_font_size;
   nux::ROProperty<bool> im_active;
   nux::ROProperty<bool> im_preedit;
+  nux::Property<bool> show_caps_lock;
 
 private:
   void OnFontChanged(GtkSettings* settings, GParamSpec* pspec=NULL);
@@ -83,9 +86,20 @@ private:
   void AddProperties(debug::IntrospectionData&);
   bool AcceptKeyNavFocus();
 
+  bool ShouldBeHighlighted();
+
+  nux::Geometry GetWaringIconGeometry() const;
+  void CheckIfCapsLockOn();
+
+  nux::ObjectPtr<nux::BaseTexture> LoadWarningIcon(int icon_size);
+  void LoadWarningTooltip();
+
+  void PaintWarningTooltip(nux::GraphicsEngine& graphics_engine);
+
 protected:
   void OnInputHintChanged();
   void OnMouseButtonDown(int x, int y, unsigned long button_flags, unsigned long key_flags);
+  void OnKeyUp(unsigned keysym, unsigned long keycode, unsigned long state);
   void OnEndKeyFocus();
 
   // getters & setters
@@ -100,20 +114,21 @@ protected:
   IMTextEntry* pango_entry_;
 
 private:
-
-  bool ShouldBeHighlighted();
-
   std::unique_ptr<nux::AbstractPaintLayer> bg_layer_;
   std::unique_ptr<nux::AbstractPaintLayer> highlight_layer_;
   nux::HLayout* layout_;
   nux::LayeredLayout* layered_layout_;
   SearchBarSpinner* spinner_;
 
+  nux::Property<bool> caps_lock_on;
   int last_width_;
   int last_height_;
+  bool mouse_over_warning_icon_;
+
+  IconTexture* warning_;
+  nux::ObjectPtr<nux::BaseTexture> warning_tooltip_;
 
   glib::SignalManager sig_manager_;
-
 };
 
 }
