@@ -22,8 +22,8 @@
 #define UNITYSHELL_SETTINGS_H
 
 #include <memory>
-#include <sigc++/signal.h>
-#include <Nux/Nux.h>
+#include <NuxCore/Property.h>
+#include "EMConverter.h"
 
 namespace unity
 {
@@ -44,10 +44,19 @@ public:
   static Settings& Instance();
   bool GetLowGfxMode() const;
   void SetLowGfxMode(const bool low_gfx);
+  EMConverter::Ptr const& em(int monitor = 0) const;
+
+  void SetLauncherWidth(int launcher_width, int monitor);
+  int LauncherWidth(int monitor) const;
 
   nux::RWProperty<FormFactor> form_factor;
   nux::Property<bool> is_standalone;
   nux::ROProperty<bool> double_click_activate;
+  nux::Property<unsigned> lim_movement_thresold;
+  nux::Property<unsigned> lim_double_click_wait;
+  nux::Property<double> font_scaling;
+
+  sigc::signal<void> dpi_changed;
 
 private:
   class Impl;
